%bayesian_urns
% % Simulation involving Bayesian inference. Urn A contains proportion p of
% red balls and Urn B contains proportion q of red balls. An urn is chosen
% at random and N balls are selected. Out of these, x are red. It is
% assumed that taking N balls does not effect the colour distribution of
% either urn.
%
% This simulation randomly chooses up to N balls from one of the urns
% (which are chosen at random), and determines the probability of each urn
% after each draw.
%
% After the first representative calculation, the simulation repeats
% M times, and the probabilities are compared to theoretical predictions
% using Bayes' theorem.
%
% LAST UPDATED by Andy French Jan 2019

function bayesian_urns

%Proportion of red balls in urns A and B
p = 0.6;
q = 0.55;

%Max number of draws
N = 1000;

%Max number of iterations
M = 1000;

%Fontsize for graphs
fsize = 18;

%

%Start clock
t0 = tic;

%Display greetings message to command window
clc;
disp(' ')
disp(' BAYESIAN URN SIMULATION')
disp(' Dr A. French. Jan 2019' );
disp(' ')
disp([' Urn A contains ',num2str(100*p),'% red balls']);
disp([' Urn B contains ',num2str(100*q),'% red balls']);
disp(' ................................... ')

%Choose urn at random, and assign probability of choosing a red ball.
r = rand;
if r>0.5
    urn = 'A';
    disp(' Urn A has been chosen.');
    P_red = p;
else
    urn = 'B';
    P_red = q;
    disp(' Urn B has been chosen.');
end

%Choose N balls from urn, the binary matrix red_balls states how many of
%these are red (1) and not red (0). Repeat this M times.
red_balls = choose_N_balls(N,M,P_red);

%Count number of red balls
x = cumsum( red_balls,1 );

%Calculate probabilities of A or B given x
n = repmat( (1:N).', [1,M] );
P_A_given_x = ( 1 + ( ( q.^x ).*(1-q).^(n-x) )./( ( p.^x ).*(1-p).^(n-x) ) ).^(-1);
P_B_given_x = ( 1 + ( ( p.^x ).*(1-p).^(n-x) )./( ( q.^x ).*(1-q).^(n-x) ) ).^(-1);

%Compute averages over M iterations
P_A_given_x_av = mean( P_A_given_x, 2 );
P_B_given_x_av = mean( P_B_given_x, 2 );

%Step through first column of red_balls, and determine the probability of
%urn A or B after each pick
for n=1:N
    %Display odds to command window
    disp(' ................................... ')
    disp( [' P(A| ',num2str(x(n,1)),' red, ',num2str(n),' draws) = ',...
        num2str(P_A_given_x(n))] );
    disp( [' P(B| ',num2str(x(n,1)),' red, ',num2str(n),' draws) = ',...
        num2str(P_B_given_x(n))] );
end

%Plot probability graphs vs draw number (first iteration)
figure('name','probability of urn A or B')
plot( 1:N, P_A_given_x(:,1).', 1:N, P_B_given_x(:,1).' );
xlabel('Ball draw number','fontsize',fsize);
ylabel('Probability of urn given data','fontsize',fsize);
title({['Probability of urn given data. Urn was actually ',urn],...
    ['p=',num2str(p),', q=',num2str(q)]},'fontsize',fsize);
grid on;
set(gca,'fontsize',fsize);
box on;
ylim([0,1])
legend({'P(A|data)','P(B|data)'},'location','best','fontsize',fsize);
print( gcf, 'Probability of urn given data.png','-dpng','-r300' );
close(gcf);

%Plot average probability graphs vs draw number
figure('name','probability of urn A or B')
plot( 1:N, P_A_given_x_av,'b-','linewidth',1); hold on
plot( 1:N, P_B_given_x_av,'g-','linewidth',1);
legend({'P(A|data)','P(B|data)'},'location','best','fontsize',fsize);
xlabel('Ball draw number','fontsize',fsize);
ylabel('Probability of urn given data','fontsize',fsize);
title({['Average probability of urn given data. ',...
    num2str(M),' iterations.'],['Urn was actually ',...
    urn,'. p=',num2str(p),', q=',num2str(q)]},'fontsize',fsize);
grid on;
set(gca,'fontsize',fsize);
box on;
ylim([0,1])
print( gcf, 'Average probability of urn given data.png','-dpng','-r300' );
close(gcf);

%Display completion message
disp(' ')
disp([' Simulation completed in ',num2str(toc(t0)),'s.'])
disp(' ');

%%

%Choose N balls
function red_balls = choose_N_balls(N,M,p)
r = rand(N,M);
red_balls = r<=p;

%End of code




